from django.shortcuts import render

from django.http import HttpResponse, HttpResponseRedirect
from django.template import Context, loader, RequestContext
from django.shortcuts import get_object_or_404, render_to_response
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from django import forms
from django.db.models import Count, Q

from .models import Expediente, Persona, Objeto, Lugar, Partida, CatastroLocal
from .gea_vars import CP, CP_DEFAULT, CP_dict, PROV, PROV_DEFAULT, CIRC, \
    CIRC_DEFAULT, NOTA, LUGAR, Lugar_dict

from django.views.generic import TemplateView, ListView, DetailView


class CounterMixin(object):

    def get_context_data(self, **kwargs):
        context = super(CounterMixin, self).get_context_data(**kwargs)
        context['count'] = self.get_queryset().count()
        return context


class SearchMixin(object):

    def get_queryset(self):
        qset = super(SearchMixin, self).get_queryset()
        q = self.request.GET.get('search')
        if q:
            q = q.split(' ')
            for w in q:
                qset = qset.filter(
                    Q(id__contains=w) |
                    Q(inscripcion_numero__contains=w) |
                    Q(expedientepartida__partida__pii__contains=w) |
                    Q(expedientepersona__persona__nombres__icontains=w) |
                    Q(expedientepersona__persona__apellidos__icontains=w) |
                    Q(expedientelugar__lugar__nombre__icontains=w)
                ).distinct()
        return qset


class ExpedienteAbiertoMixin(object):

    def get_queryset(self):
        qset = super(ExpedienteAbiertoMixin, self).get_queryset()
        qset = qset.filter(
            Q(inscripcion_numero__isnull=True) &
            Q(cancelado=False) &
            Q(sin_inscripcion=False)
        )[:10]
        return qset


class Home(CounterMixin, SearchMixin, ExpedienteAbiertoMixin, ListView):
    template_name = 'home/index.html'
    model = Expediente

    def get_context_data(self, *args, **kwargs):
        context = super(Home, self).get_context_data(*args, **kwargs)
        # last 5 inscriptos
        context['insc_list'] = Expediente.objects.filter(Q(inscripcion_numero__isnull=False) &
                                                         Q(inscripcion_fecha__isnull=False)).order_by('-inscripcion_fecha', 'inscripcion_numero')[:5]
        # ordenes pendientes
        context['ord_list'] = Expediente.objects.filter(Q(orden_numero__isnull=False) &
                                                        Q(inscripcion_numero__isnull=True) &
                                                        Q(orden_fecha__isnull=False)).order_by('-orden_fecha', 'orden_numero')[:5]
        # relevamientos
        context['relev_list'] = Expediente.objects.filter(fecha_medicion__isnull=False).order_by('-fecha_medicion')[:5]
        # altas
        context['open_list'] = Expediente.objects.filter(created__isnull=False).order_by('-created')[:10]
        return context

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(Home, self).dispatch(*args, **kwargs)


class About(TemplateView):
    template_name = 'home/about.html'


class ExpedienteMixin(object):

    def get_queryset(self):
        qset = super(ExpedienteMixin, self).get_queryset()
        q = self.request.GET.get('pendiente')
        if q:  # orden pendiente
            qset = qset.filter(
                Q(inscripcion_numero__isnull=q) &
                Q(orden_numero__isnull=not q)
            )
        q = self.request.GET.get('inscripto')
        if q:  # plano inscripto
            qset = qset.filter(inscripcion_numero__isnull=False)
        else:
            qset = qset.filter(inscripcion_numero__isnull=True)
        q = self.request.GET.get('cancelado')
        if q:  # expediente cancelado
            qset = qset.filter(cancelado=q)
        q = self.request.GET.get('duplicado')
        if q:  # duplicado
            qset = qset.filter(duplicado=q)
        q = self.request.GET.get('sin_inscr')
        if q:  # sin inscripcion
            qset = qset.filter(sin_inscripcion=q)
        return qset


class ExpedienteList(CounterMixin, SearchMixin, ListView):
    template_name = 'lists/expediente_list.html'
    model = Expediente
    paginate_by = 10

    def get_paginate_by(self, queryset):
        """
        Paginate by specified value in querystring, or use default class
        property value.
        """
        return self.request.GET.get('paginate_by', self.paginate_by)

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(ExpedienteList, self).dispatch(*args, **kwargs)


class ExpedienteDetail(DetailView):
    template_name = 'lists/expediente_detail.html'
    model = Expediente

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(ExpedienteDetail, self).dispatch(*args, **kwargs)


class NombreSearchMixin(object):

    def get_queryset(self):
        qset = super(NombreSearchMixin, self).get_queryset()
        q = self.request.GET.get('search')
        if q:
            q = q.split(' ')
            for w in q:
                qset = qset.filter(
                    Q(nombres__icontains=w) |
                    Q(apellidos__icontains=w) |
                    Q(nombres_alternativos__icontains=w) |
                    Q(apellidos_alternativos__icontains=w) |
                    Q(expedientepersona__expediente__id__contains=w) |
                    Q(expedientepersona__expediente__inscripcion_numero__contains=w) |
                    Q(expedientepersona__expediente__expedientepartida__partida__pii__contains=w) |
                    Q(expedientepersona__expediente__expedientelugar__lugar__nombre__icontains=w)
                ).distinct()
        return qset


class PersonaList(CounterMixin, NombreSearchMixin, ListView):
    template_name = 'lists/persona_list.html'
    model = Persona
    paginate_by = 50

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(PersonaList, self).dispatch(*args, **kwargs)


class PersonaDetail(DetailView):
    template_name = 'lists/persona_detail.html'
    model = Persona

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(PersonaDetail, self).dispatch(*args, **kwargs)


class LugarSearchMixin(object):

    def get_queryset(self):
        queryset = super(LugarSearchMixin, self).get_queryset()
        q = self.request.GET.get('lugar')
        if q:
            return queryset.filter(expedientelugar__lugar__nombre=q)
        return queryset


class SeccionSearchMixin(object):

    def get_queryset(self):
        queryset = super(SeccionSearchMixin, self).get_queryset()
        q = self.request.GET.get('seccion')
        if q:
            return queryset.filter(expedientelugar__catastrolocal__seccion=q)
        return queryset


class ManzanaSearchMixin(object):

    def get_queryset(self):
        queryset = super(ManzanaSearchMixin, self).get_queryset()
        q = self.request.GET.get('manzana')
        if q:
            return queryset.filter(expedientelugar__catastrolocal__manzana=q)
        return queryset


class ParcelaSearchMixin(object):

    def get_queryset(self):
        queryset = super(ParcelaSearchMixin, self).get_queryset()
        q = self.request.GET.get('parcela')
        if q:
            return queryset.filter(expedientelugar__catastrolocal__parcela=q)
        return queryset


class CLMixin(object):

    def get_queryset(self):
        qset = super(CLMixin, self).get_queryset()
        q = self.request.GET.get('lugar')
        if q is not None:
            qset = qset.filter(expedientelugar__lugar__nombre=q).distinct()
        q = self.request.GET.get('seccion')
        if q is not None:
            qset = qset.filter(expedientelugar__catastrolocal__seccion=q)
        q = self.request.GET.get('manzana')
        if q is not None:
            qset = qset.filter(expedientelugar__catastrolocal__manzana=q)
        q = self.request.GET.get('parcela')
        if q is not None:
            qset = qset.filter(expedientelugar__catastrolocal__parcela=q)
        return qset


class CatastroLocalList(CounterMixin, CLMixin, ListView):
    template_name = 'lists/catastros_locales.html'
    model = Expediente
    paginate_by = 10

    def get_context_data(self, **kwargs):
        context = super(CatastroLocalList, self).get_context_data(**kwargs)
        l = self.request.GET.get('lugar')
        context['lugar'] = l
        s = self.request.GET.get('seccion')
        context['seccion'] = s
        m = self.request.GET.get('manzana')
        context['manzana'] = m
        p = self.request.GET.get('parcela')
        context['parcela'] = p
        context['lugares'] = Lugar.objects.values_list('nombre',
          flat=True).order_by('nombre')
        context['secciones'] = CatastroLocal.objects.filter(
          expediente_lugar__lugar__nombre=l).values_list('seccion',
          flat=True).distinct().order_by('seccion')
        context['manzanas'] = CatastroLocal.objects.filter(
          expediente_lugar__lugar__nombre=l).filter(seccion=s).values_list(
          'manzana', flat=True).distinct().order_by('manzana')
        context['parcelas'] = CatastroLocal.objects.filter(
          expediente_lugar__lugar__nombre=l).filter(Q(seccion=s) &
          Q(manzana=m)).values_list(
          'parcela', flat=True).distinct().order_by('parcela')
        return context

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(CatastroLocalList, self).dispatch(*args, **kwargs)


class CaratulaForm(forms.Form):
    expte_nro = forms.IntegerField(label='Expediente Nº',
        widget=forms.NumberInput(attrs={'placeholder': 'ej: 4300'}))
    inmueble = forms.CharField(widget=forms.Textarea(attrs={
        'placeholder': 'ej: Una fracción de terreno...'
        }), required=False)
    tomo = forms.IntegerField(required=False)
    par = forms.BooleanField(required=False)
    folio = forms.IntegerField(required=False)
    numero = forms.IntegerField(label='Número', required=False)
    fecha = forms.DateField(required=False)
    obs = forms.CharField(label='Observaciones',
                          widget=forms.Textarea(attrs={
                              'placeholder': 'ej: Modifica el Lote Nº 1 del Plano Nº 23.456. Etc.'
                          }), required=False)
    FORMAT_CHOICES = (('html', 'HTML'),('dxf', 'DXF'),)
    fmt = forms.ChoiceField(label='Formato de salida', choices=FORMAT_CHOICES)


@login_required
def caratula(request):
    if request.method == 'POST':  # If the form has been submitted...
        # CaratulaForm was defined in the previous section
        form = CaratulaForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            # Process the data in form.cleaned_data
            #
            expediente_id = form.cleaned_data['expte_nro']
            inmueble = form.cleaned_data['inmueble']
            tomo = form.cleaned_data['tomo']
            par = form.cleaned_data['par']
            folio = form.cleaned_data['folio']
            numero = form.cleaned_data['numero']
            fecha = form.cleaned_data['fecha']
            obs = form.cleaned_data['obs']
            fmt = form.cleaned_data['fmt']

            e = get_object_or_404(Expediente, id=expediente_id)
            template = loader.get_template('tools/caratula.%s' % fmt)
            context = Context({
                'e': e,
                'inmueble': inmueble,
                'tomo': tomo,
                'par': par,
                'folio': folio,
                'numero': numero,
                'fecha_dominio': fecha,
                'obs': obs,
                'fmt': fmt,
            })
            if fmt != "html":
                response = HttpResponse(template.render(context), content_type='image/x-%s' % fmt)
                response['Content-Disposition'] = 'attachment; filename=%04d-caratula.%s' % (expediente_id, fmt)
            else:
                return HttpResponse(template.render(context))
            return response
    else:
        form = CaratulaForm()  # An unbound form

    return render(request, 'tools/caratula_form.html', {
        'form': form,
    })


class SolicitudForm(forms.Form):
    expte_nro = forms.IntegerField(label='Expediente Nº',
        widget=forms.NumberInput(attrs={'placeholder': 'ej: 4300'}))
    circunscripcion = forms.ChoiceField(label='Circunscripción', choices=CIRC, initial=CIRC_DEFAULT)
    domicilio_fiscal = forms.CharField(max_length=40, widget=forms.TextInput(
        attrs={'placeholder': 'ej: San Martín 430'}))
    localidad = forms.ChoiceField(choices=CP, initial=CP_DEFAULT)
    provincia = forms.ChoiceField(choices=PROV, initial=PROV_DEFAULT)
    nota_titulo = forms.ChoiceField(label='Nota título', choices=NOTA)
    nota = forms.CharField(label='Nota contenido',
        widget=forms.Textarea(attrs={
        'placeholder': 'Ingrese el texto de la nota correspondiente'
        }), required=False)


@login_required
def solicitud(request):
    if request.method == 'POST':  # If the form has been submitted...
        # SolicitudForm was defined in the previous section
        form = SolicitudForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            # Process the data in form.cleaned_data
            # ...
            expediente_id = form.cleaned_data['expte_nro']
            circunscripcion = form.cleaned_data['circunscripcion']
            domicilio_fiscal = form.cleaned_data['domicilio_fiscal']
            # codigo_postal = form.cleaned_data['localidad']
            loc = form.cleaned_data['localidad']
            localidad = CP_dict[loc].split(' - ')[0]
            codigo_postal = CP_dict[loc].split(' - ')[1]
            provincia = form.cleaned_data['provincia']
            nota_titulo = form.cleaned_data['nota_titulo']
            nota = form.cleaned_data['nota']

            e = get_object_or_404(Expediente, id=expediente_id)
            template = loader.get_template('doc/solic.html')
            context = Context({
                'e': e,
                'domfiscal': domicilio_fiscal,
                'circunscripcion': circunscripcion,
                'localidad': localidad,
                'cp': codigo_postal,
                'provincia': provincia,
                'nota_titulo': nota_titulo,
                'nota': nota,
            })
            return HttpResponse(template.render(context))
    else:
        form = SolicitudForm()  # An unbound form

    return render(request, 'doc/solic_form.html', {
        'form': form,
    })


class VisacionForm(forms.Form):
    expte_nro = forms.IntegerField(label='Expediente Nº')
    lugar = forms.ChoiceField(choices=LUGAR, initial=0)


@login_required
def visacion(request):
    if request.method == 'POST':  # If the form has been submitted...
        # VisacionForm was defined in the previous section
        form = VisacionForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            # Process the data in form.cleaned_data
            # ...
            eid = form.cleaned_data['expte_nro']
            lugar = form.cleaned_data['lugar']
            sr = Lugar_dict[int(lugar)][0]
            localidad = Lugar_dict[int(lugar)][1]

            e = get_object_or_404(Expediente, id=eid)
            # Redirect after POST
            return render_to_response('doc/visac.html', {
                "e": e,
                "sr": sr,
                "localidad": localidad
            }, context_instance=RequestContext(request))
    else:
        form = VisacionForm()  # An unbound form

    return render(request, 'doc/visac_form.html', {
        'form': form,
    })


#
# Buscar Plano por Nro
#
ftp_url = 'ftp://zentyal.estudio.lan'


class PlanoForm(forms.Form):
    circ = forms.IntegerField(label='Circunscripción', min_value=1, max_value=2, initial=1)
    n_insc = forms.IntegerField(label='Plano Nº', min_value=1, max_value=999999)


@login_required
def plano(request):
    if request.method == 'POST':  # If the form has been submitted...
        form = PlanoForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            circ = form.cleaned_data['circ']
            nro = form.cleaned_data['n_insc']
            return HttpResponseRedirect(
                '%s/planos/%s/%06d.pdf' % (ftp_url, circ, nro))
    else:
        form = PlanoForm()  # An unbound form

    return render(request, 'search/plano_form.html', {
        'form': form,
    })


#
# Buscar Set de Datos por PII
#
class SetForm(forms.Form):
    partida = forms.IntegerField(min_value=1, max_value=999999)
    sub_pii = forms.IntegerField(label='Subpartida', min_value=0, max_value=9999, initial=0)


@login_required
def set(request):
    if request.method == 'POST':  # If the form has been submitted...
        form = SetForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            pii = form.cleaned_data['partida']
            sub_pii = form.cleaned_data['sub_pii']
            return HttpResponseRedirect(
                '%s/set/%06d%04d.pdf' % (ftp_url, pii, sub_pii))
    else:
        form = SetForm()  # An unbound form

    return render(request, 'search/set_form.html', {
        'form': form,
    })


#
# Calcular Digito Verificador de la PII
#
class DVAPIForm(forms.Form):
    dp = forms.IntegerField(label='DP (departamento)', min_value=1, max_value=19, initial=11)
    ds = forms.IntegerField(label='DS (distrito)', min_value=1, max_value=99, initial=8)
    sd = forms.IntegerField(label='SD (subdistrito)', min_value=0, max_value=99, initial=0)
    partida = forms.IntegerField(min_value=1, max_value=999999)
    sub_pii = forms.IntegerField(label='Subpartida', min_value=0, max_value=9999, initial=0)


def get_dvapi(dp, ds, sd, pii, subpii):
    coef = '9731'
    _coef = coef + coef + coef + coef
    strpii = '%02d%02d%02d%06d%04d' % (dp, ds, sd, pii, subpii)
    suma = 0
    for i in range(0, len(strpii)):
        m = str(int(strpii[i]) * int(_coef[i]))
        suma += int(m[len(m) - 1])
    return (10 - (suma % 10)) % 10


def dvapi(request):
    if request.method == 'POST':  # If the form has been submitted...
        form = DVAPIForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            dp = form.cleaned_data['dp']
            ds = form.cleaned_data['ds']
            sd = form.cleaned_data['sd']
            pii = form.cleaned_data['partida']
            sub_pii = form.cleaned_data['sub_pii']
            dv = get_dvapi(dp, ds, sd, pii, sub_pii)
            return render_to_response('tools/dvapi_form.html', {
                'dv': dv,
                'form': form
            }, context_instance=RequestContext(request))
    else:
        form = DVAPIForm()  # An unbound form

    return render(request, 'tools/dvapi_form.html', {
        'form': form,
    })


#
# Consultar estado en Sistema de Información de Expedientes del SCIT
#
class SIEForm(forms.Form):
    mesa = forms.IntegerField(min_value=13401, max_value=13401, initial=13401)
    nro = forms.IntegerField(label='Número', min_value=1, max_value=9999999)
    digito = forms.IntegerField(label='Dígito', min_value=0, max_value=9, initial=0)


def sie(request):
    if request.method == 'POST':  # If the form has been submitted...
        form = SIEForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            mesa = form.cleaned_data['mesa']
            nro = form.cleaned_data['nro']
            dv = form.cleaned_data['digito']
            #dv = get_dvsie(mesa, nro)
            return HttpResponseRedirect(
                'https://www.santafe.gov.ar/index.php/apps/sie?mesa=%d&numero=%d&digito=%d' % (mesa, nro, dv))
    else:
        form = SIEForm()  # An unbound form

    return render(request, 'tools/sie_form.html', {
        'form': form,
    })


#
#
# Exptes x Catastro Local
#
#
class CLForm(forms.Form):
    lugar = forms.ModelChoiceField(queryset=Lugar.objects.exclude(
                nombre__startswith='Colonia').exclude(
                nombre__startswith='Zona Rural').exclude(
                nombre__startswith='Zona de Islas'), required=False)
    seccion = forms.CharField(max_length=4, required=False)
    manzana = forms.CharField(max_length=4, required=False)
    parcela = forms.CharField(max_length=4, required=False)


@login_required
def catastro(request):
    if request.method == 'POST':  # If the form has been submitted...
        # VisacionForm was defined in the previous section
        form = CLForm(request.POST)  # A form bound to the POST data
        if form.is_valid():  # All validation rules pass
            # Process the data in form.cleaned_data
            # ...
            lugar = form.cleaned_data['lugar']
            seccion = form.cleaned_data['seccion']
            manzana = form.cleaned_data['manzana']
            parcela = form.cleaned_data['parcela']

            filtro = u'?'
            if lugar is not None:
                filtro = u'%s%s%s' % (
                    filtro, u'&expedientelugar__lugar__nombre=', lugar)
            if seccion != '':
                filtro = u'%s%s%s' % (
                    filtro,
                    u'&expedientelugar__catastrolocal__seccion=',
                    seccion
                )
            if manzana != '':
                filtro = u'%s%s%s' % (
                    filtro,
                    u'&expedientelugar__catastrolocal__manzana=',
                    manzana
                )
            if parcela != '':
                filtro = u'%s%s%s' % (
                    filtro,
                    u'&expedientelugar__catastrolocal__parcela=',
                    parcela
                )
            # Redirect after POST
            return HttpResponseRedirect('/admin/gea/expediente/%s' % filtro)
    else:
        form = CLForm()  # An unbound form

    return render(request, 'search/catastro_form.html', {
        'form': form,
    })


######
###### EXPERIMENTAL
######
from calendar import HTMLCalendar
from datetime import date
from itertools import groupby

from django.utils.html import conditional_escape as esc

class QuerysetCalendar(HTMLCalendar):

    def __init__(self, queryset, datefield):
        self.datefield = datefield
        super(QuerysetCalendar, self).__init__()
        self.queryset_by_date = self.group_by_day(queryset)

    def formatday(self, day, weekday):
        if day != 0:
            cssclass = self.cssclasses[weekday]
            if date.today() == date(self.year, self.month, day):
                cssclass += ' today'
            if day in self.queryset_by_date:
                cssclass += ' filled'
                body = ['<ul>']
                for item in self.queryset_by_date[day]:
                    body.append('<li>')
                    body.append('<a href="%s">' % item.get_absolute_url())
                    body.append(esc(item))
                    body.append('</a></li>')
                body.append('</ul>')
                return self.day_cell(cssclass, '%d %s' % (day, ''.join(body)))
            return self.day_cell(cssclass, day)
        return self.day_cell('noday', ' ')


    def formatmonth(self, year, month):
        self.year, self.month = year, month
        return super(QuerysetCalendar, self).formatmonth(year, month)

    def group_by_day(self, queryset):
        field = lambda item: getattr(item, self.datefield).day
        return dict(
            [(day, list(items)) for day, items in groupby(queryset, field)]
        )

    def day_cell(self, cssclass, body):
        return '<td class="%s">%s</td>' % (cssclass, body)


#######


from django.utils.safestring import mark_safe


@login_required
def calendar(request, year, month):
    e = Expediente.objects.order_by('fecha_medicion').filter(
        fecha_medicion__year=year, fecha_medicion__month=month
        )
    cal = QuerysetCalendar(e, 'fecha_medicion').formatmonth(int(year), int(month))
    return render_to_response('tools/calendar.html', {'calendar': mark_safe(cal),})



from django.db import models
from django_extensions.db.models import (
    TitleSlugDescriptionModel, TimeStampedModel)
# from django.urls import reverse
from django.core.urlresolvers import reverse
from filebrowser.fields import FileBrowseField


def CapitalizePhrase(string):
    phrase = ''
    for word in string.split():
        phrase = u'%s %s%s' % (phrase, word[0].upper(), word[1:].lower())
    return phrase.strip()


class CharNullField(models.CharField):  # subclass the CharField
    description = "CharField that stores NULL but returns ''"
    # this ensures to_python will be called

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return u''
        return value

    # this is the value right out of the db, or an instance
    def to_python(self, value):
        # if an instance, just return the instance
        if isinstance(value, models.CharField):
            return value
        if value is None:  # if the db has a NULL (==None in Python)
            return u''  # convert it into the Django-friendly '' string
        else:
            return value  # otherwise, return just the value

    # catches value right before sending to db
    def get_prep_value(self, value):
        # if Django tries to save '' string, send the db None (NULL)
        if value == u'':
            return None
        else:
            return value  # otherwise, just pass the value


class URLNullField(models.URLField):  # subclass the URLField
    description = "URLField that stores NULL but returns ''"
    # this ensures to_python will be called

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return u''
        return value

    # this is the value right out of the db, or an instance
    def to_python(self, value):
        # if an instance, just return the instance
        if isinstance(value, models.URLField):
            return value
        if value is None:  # if the db has a NULL (==None in Python)
            return u''  # convert it into the Django-friendly '' string
        else:
            return value  # otherwise, return just the value

    # catches value right before sending to db
    def get_prep_value(self, value):
        # if Django tries to save '' string, send the db None (NULL)
        if value == u'':
            return None
        else:
            return value  # otherwise, just pass the value


class EmailNullField(models.EmailField):  # subclass the EmailField
    description = "EmailField that stores NULL but returns ''"
    # this ensures to_python will be called

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return u''
        return value

    # this is the value right out of the db, or an instance
    def to_python(self, value):
        # if an instance, just return the instance
        if isinstance(value, models.EmailField):
            return value
        if value is None:  # if the db has a NULL (==None in Python)
            return u''  # convert it into the Django-friendly '' string
        else:
            return value  # otherwise, return just the value

    # catches value right before sending to db
    def get_prep_value(self, value):
        # if Django tries to save '' string, send the db None (NULL)
        if value == u'':
            return None
        else:
            return value  # otherwise, just pass the value


class Antecedente(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(
        'Expediente', null=True, blank=True, default=None)
    expediente_modificado = models.ForeignKey(
        'Expediente', null=True, blank=True, default=None,
        related_name='expediente_modificado')
    inscripcion_numero = models.IntegerField(
        null=True, blank=True, default=None)
    duplicado = models.BooleanField(default=False)
    obs = CharNullField("Observaciones", max_length=255, null=True, blank=True,
                        default=None)
    plano_ruta = URLNullField(
        max_length=100, null=True, blank=True, default=None)
    plano = FileBrowseField("Enlace al plano", max_length=200,
                            directory="planos/", extensions=[".pdf"],
                            blank=True, null=True)


class Catastro(models.Model):
    id = models.AutoField(primary_key=True)
    expediente_partida = models.ForeignKey('ExpedientePartida')
    zona = models.ForeignKey(
        'Zona', db_column='zona', null=True, blank=True, default=None)
    seccion = CharNullField(max_length=10, null=True, blank=True, default=None)
    poligono = CharNullField(
        max_length=10, null=True, blank=True, default=None)
    manzana = CharNullField(max_length=10, null=True, blank=True, default=None)
    parcela = CharNullField(max_length=10, null=True, blank=True, default=None)
    subparcela = CharNullField(
        max_length=10, null=True, blank=True, default=None)

    def __str__(self):
        if str(self.zona) in ('1', '2', '3'):
            return u''.join(('Z:', str(self.zona), ' - S:', self.seccion,
                             ' - M:', self.manzana, ' - P:', self.parcela))
        elif str(self.zona) in ('4', '5'):
            return u''.join(('Z:', str(self.zona), ' - Pol:', self.poligono,
                             ' - P:', self.parcela))
        else:
            return u''


class CatastroLocal(models.Model):
    id = models.AutoField(primary_key=True)
    expediente_lugar = models.ForeignKey('ExpedienteLugar')
    seccion = CharNullField(max_length=20, null=True, blank=True, default=None)
    manzana = CharNullField(max_length=20, null=True, blank=True, default=None)
    parcela = CharNullField(max_length=20, null=True, blank=True, default=None)
    subparcela = CharNullField(
        max_length=20, null=True, blank=True, default=None)
    suburbana = models.BooleanField(default=False)
    poligono = CharNullField(
        max_length=20, null=True, blank=True, default=None)

    class Meta:
        verbose_name = 'catastro local'
        verbose_name_plural = 'catastros locales'

    def __str__(self):
        return u''.join(('S:', self.seccion, ' - M:', self.manzana, ' - P:',
                         self.parcela))


class Circunscripcion(models.Model):
    id = models.IntegerField(primary_key=True)
    nombre = CharNullField(max_length=10)
    orden = CharNullField(max_length=7)

    class Meta:
        ordering = ['id']
        verbose_name_plural = 'circunscripciones'

    def __str__(self):
        return self.nombre


class Comprobante(models.Model):
    id = models.AutoField(primary_key=True)
    nombre = CharNullField(max_length=50)

    def __str__(self):
        return self.nombre


class Presupuesto(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey('Expediente')
    monto = models.DecimalField(max_digits=8, decimal_places=2)
    fecha = models.DateField()
    porcentaje_cancelado = models.DecimalField(
        max_digits=5, decimal_places=2, verbose_name='% cancelado')
    obs = CharNullField("Observaciones", max_length=255, null=True, blank=True,
                        default=None)

    class Meta:
        ordering = ['expediente']

    def __str__(self):
        return u'%s - $%s - %s' % (self.expediente, str(self.monto),
                                   str(self.fecha))


class Pago(models.Model):
    id = models.AutoField(primary_key=True)
    presupuesto = models.ForeignKey(Presupuesto)
    comprobante = models.ForeignKey(Comprobante)
    comprobante_nro = models.IntegerField(null=True, blank=True, default=None)
    fecha = models.DateField(null=True, blank=True, default=None)
    monto = models.DecimalField(max_digits=8, decimal_places=2)
    porcentaje = models.DecimalField(max_digits=5, decimal_places=2)
    obs = CharNullField("Observaciones", max_length=255, null=True, blank=True,
                        default=None)

    def __str__(self):
        return u'%s%% - %s' % (str(self.porcentaje), str(self.fecha))


class Dp(models.Model):
    dp = models.IntegerField(primary_key=True)
    nombre = CharNullField(max_length=50, verbose_name='nombre depto')
    habitantes = models.IntegerField(null=True, blank=True)
    superficie = models.IntegerField(null=True, blank=True)
    cabecera = CharNullField(max_length=50, null=True, blank=True)
    circunscripcion = models.ForeignKey(Circunscripcion)

    def departamento(self):
        return u'%02d %s' % (self.dp, self.nombre)

    class Meta:
        verbose_name_plural = 'Departamentos'
        ordering = ['dp']

    def __str__(self):
        return u'%02d' % self.dp


class Ds(models.Model):
    id = models.AutoField(primary_key=True)
    dp = models.ForeignKey(Dp, db_column='dp')
    ds = models.IntegerField()
    nombre = CharNullField(max_length=50, verbose_name='nombre distrito')

    def distrito(self):
        return u'%02d' % self.ds

    class Meta:
        verbose_name_plural = 'distritos'
        ordering = ['dp', 'ds']

    def __str__(self):
        return u'%02d' % self.ds


class Sd(models.Model):
    id = models.AutoField(primary_key=True)
    ds = models.ForeignKey(Ds, db_column='ds')
    sd = models.IntegerField()
    nombre = CharNullField(
        max_length=50, null=True, blank=True, default=None,
        verbose_name='nombre subdistrito')

    def subdistrito(self):
        return u'%02d' % self.sd

    def dp(self):
        return self.ds.dp
    dp.short_description = 'Dp'

    def dp_nombre(self):
        return self.ds.dp.nombre
    dp_nombre.short_description = 'Dp nombre'

    def ds_nombre(self):
        return self.ds.nombre
    ds_nombre.short_description = 'Ds nombre'

    class Meta:
        verbose_name_plural = 'subdistritos'
        ordering = ['ds', 'sd']

    def __str__(self):
        return u'%s%s%02d' % (self.ds.dp, self.ds, self.sd)
    nomenclatura = property(__str__)


class Expediente(TimeStampedModel):
    id = models.IntegerField('Expediente Nº', primary_key=True)
    fecha_plano = models.DateField(null=True, blank=True, default=None)
    fecha_medicion = models.DateField(null=True, blank=True, default=None)
    inscripcion_numero = models.IntegerField('SCIT inscripción Nº',unique=True,
                                             null=True, blank=True,
                                             default=None)
    inscripcion_fecha = models.DateField('Fecha inscripción', null=True,
                                         blank=True, default=None)
    duplicado = models.BooleanField(default=False)
    orden_numero = models.IntegerField('CoPA Expendiente Nº', null=True,
                                       blank=True, default=None)
    orden_fecha = models.DateField('Fecha contrato', null=True, blank=True,
                                   default=None)
    sin_inscripcion = models.BooleanField(default=False)
    cancelado = models.BooleanField(default=False)
    cancelado_por = CharNullField(max_length=100, null=True, blank=True,
                                  default=None)
    plano_ruta = URLNullField(max_length=100, null=True, blank=True,
                              default=None)
    plano = FileBrowseField("Enlace al plano", max_length=200,
                            directory="planos/", extensions=[".pdf"],
                            blank=True, null=True)

    def get_absolute_url(self):
        return reverse('expediente', kwargs={'pk': str(self.id)})
        # return "/gea/expedientes/%i" % self.id # MAL: poco portable

    def inscripto(self):
        return (self.inscripcion_numero != 0)

    def propietarios_count(self):
        """Devuelve la cantidad de personas que figuran como propietarias."""
        return self.expedientepersona_set.filter(propietario=True).count()

    class Meta:
        ordering = ['-id']

    def __str__(self):
        return u'%d' % self.id


class ExpedienteLugar(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(Expediente)
    lugar = models.ForeignKey('Lugar')

    class Meta:
        verbose_name = 'lugar'
        verbose_name_plural = 'lugares'
        ordering = ['expediente', 'lugar']

    def __str__(self):
        return self.lugar.nombre


class ExpedienteObjeto(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(Expediente)
    objeto = models.ForeignKey('Objeto')

    class Meta:
        verbose_name = 'objeto'
        verbose_name_plural = 'objetos'
        ordering = ['expediente', 'objeto']

    def __str__(self):
        return self.objeto.nombre


class ExpedientePartida(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(Expediente)
    partida = models.ForeignKey('Partida')
    set_ruta = URLNullField(
        max_length=100, null=True, blank=True, default=None)
    informe_catastral = FileBrowseField(max_length=200, directory="set/",
                                        extensions=[".pdf"], blank=True,
                                        null=True)

    class Meta:
        verbose_name = 'partida'
        verbose_name_plural = 'partidas'
        ordering = ['expediente', 'partida']

    def __str__(self):
        return u'%s' % self.partida


class ExpedientePersona(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(Expediente)
    persona = models.ForeignKey('Persona')
    comitente = models.BooleanField(default=False)
    propietario = models.BooleanField(default=True)
    poseedor = models.BooleanField(default=False)
    sucesor = models.BooleanField(default=False)
    partes_indivisas_propias = models.IntegerField(
        null=True, blank=True, default=None)
    partes_indivisas_total = models.IntegerField(
        null=True, blank=True, default=None)
    sucesion = models.BooleanField(default=False)
    nuda_propiedad = models.BooleanField(default=False)
    usufructo = models.BooleanField(default=False)

    class Meta:
        verbose_name = 'persona involucrada'
        verbose_name_plural = 'personas involucradas'
        ordering = [
            'persona__apellidos', 'persona__nombres']

    def __str__(self):
        return u'%d - %s %s' % (self.expediente.id,
                                self.persona.apellidos,
                                self.persona.nombres)


class ExpedienteProfesional(models.Model):
    id = models.AutoField(primary_key=True)
    expediente = models.ForeignKey(Expediente)
    profesional = models.ForeignKey('Profesional')

    class Meta:
        verbose_name = 'profesional firmante'
        verbose_name_plural = 'profesionales firmantes'
        ordering = ['profesional__apellidos', 'profesional__nombres']

    def __str__(self):
        return self.profesional.nombre_completo()


class Lugar(models.Model):
    id = models.AutoField(primary_key=True)
    nombre = CharNullField(max_length=80)
    obs = CharNullField("Observaciones", max_length=255, null=True, blank=True,
                        default=None)

    class Meta:
        verbose_name_plural = 'lugares'
        ordering = ['nombre']

    def __str__(self):
        return self.nombre


class Objeto(models.Model):
    id = models.AutoField(primary_key=True)
    nombre = CharNullField(max_length=80)

    class Meta:
        ordering = ['nombre']

    def __str__(self):
        return self.nombre


class Partida(models.Model):
    id = models.AutoField(primary_key=True)
    sd = models.ForeignKey(
        'Sd', db_column='sd', null=True, blank=True, default=None)
    pii = models.IntegerField()
    subpii = models.IntegerField()
    api = models.SmallIntegerField(null=True, blank=True, default=0)

    def pii_completa(self):
        return u'%s-%06d/%04d-%d' % (self.sd, self.pii, self.subpii, self.api)

    def calc_dvapi(self, sd, pii, subpii=0):
        coef = '9731'
        _coef = coef + coef + coef + coef
        strpii = '%06d%06d%04d' % (sd, pii, subpii)
        suma = 0
        for i in range(0, len(strpii)):
            m = str(int(strpii[i]) * int(_coef[i]))
            suma += int(m[len(m) - 1])
        return (10 - (suma % 10)) % 10

    def get_dvapi(self):
        return self.calc_dvapi(int(self.sd.nomenclatura), self.pii,
                               self.subpii)
    dvapi = property(get_dvapi)

    class Meta:
        unique_together = (('pii', 'subpii'))
        ordering = ['pii', 'subpii']

    def __str__(self):
        return u'%06d/%04d-%d' % (self.pii, self.subpii, self.api)

    def save(self, force_insert=False, force_update=False):
        self.subpii = self.subpii if self.subpii else 0
        self.api = self.get_dvapi()
        super(Partida, self).save(force_insert, force_update)


class PartidaDominio(models.Model):
    id = models.AutoField(primary_key=True)
    partida = models.ForeignKey(Partida)
    tomo = models.IntegerField(null=True, blank=True, default=None)
    par = models.BooleanField(default=False)
    impar = models.BooleanField(default=False)
    folio = models.IntegerField(null=True, blank=True, default=None)
    numero = models.IntegerField(null=True, blank=True, default=None)
    fecha = models.DateField(null=True, blank=True, default=None)
    fecha_inscripcion_definitiva = models.DateField(
        null=True, blank=True, default=None)

    class Meta:
        verbose_name_plural = 'partida_dominios'


class Persona(models.Model):
    id = models.AutoField(primary_key=True)
    nombres = CharNullField(max_length=100, null=True, blank=True,
                            default=None)
    apellidos = CharNullField(max_length=100)
    nombres_alternativos = CharNullField(
        max_length=100, null=True, blank=True, default=None)
    apellidos_alternativos = CharNullField(
        max_length=100, null=True, blank=True, default=None)
    domicilio = CharNullField(
        max_length=50, null=True, blank=True, default=None)
    lugar = models.ForeignKey(Lugar, null=True, blank=True, default=None)
    telefono = CharNullField(max_length=20, null=True, blank=True,
                             default=None)
    celular = CharNullField(max_length=20, null=True, blank=True, default=None)
    email = EmailNullField(
        max_length=50, unique=True, null=True, blank=True, default=None)
    cuit_cuil = CharNullField(
        max_length=14, unique=True, null=True, blank=True, default=None,
        verbose_name='CUIT/CUIL/CDI')
    TIPO_DOC = ((0, 'DNI'), (1, 'LC'), (2, 'LE'), (3, 'Otro'))
    tipo_doc = models.SmallIntegerField(
        null=True, blank=True, choices=TIPO_DOC, default=None)
    documento = models.IntegerField(
        unique=True, null=True, blank=True, default=None)

    def get_absolute_url(self):
        return reverse('persona', kwargs={'pk': str(self.id)})

    def nombre_completo(self):
        return (u'%s %s' % (self.apellidos, self.nombres)).strip()

    def show_tipo_doc(self):
        if self.tipo_doc != '' and self.tipo_doc is not None:
            return self.TIPO_DOC[self.tipo_doc][1]
    show_tipo_doc.short_description = 'Tipo doc'
    show_tipo_doc.admin_order_field = 'tipo_doc'

    class Meta:
        ordering = ['apellidos', 'nombres']

    def __str__(self):
        return self.nombre_completo()

    def save(self, force_insert=False, force_update=False):
        self.apellidos = self.apellidos.upper().strip()
        self.apellidos_alternativos = self.apellidos_alternativos.upper().strip()
        self.nombres = CapitalizePhrase(self.nombres)
        self.nombres_alternativos = CapitalizePhrase(self.nombres_alternativos)
        super(Persona, self).save(force_insert, force_update)


class Profesional(models.Model):
    id = models.AutoField(primary_key=True)
    nombres = CharNullField(max_length=50)
    apellidos = CharNullField(max_length=50)
    titulo = models.ForeignKey('Titulo', null=True, blank=True, default=None)
    icopa = CharNullField(max_length=8, null=True, blank=True, default=None)
    domicilio = CharNullField(
        max_length=50, null=True, blank=True, default=None)
    lugar = models.ForeignKey(Lugar, null=True, blank=True, default=None)
    telefono = CharNullField(
        max_length=20, null=True, blank=True, default=None)
    celular = CharNullField(max_length=20, null=True, blank=True, default=None)
    web = URLNullField(max_length=50, null=True, blank=True, default=None)
    email = EmailNullField(
        max_length=50, unique=True, null=True, blank=True, default=None)
    cuit_cuil = CharNullField(
        max_length=14, unique=True, null=True, blank=True, default=None,
        verbose_name='DNI/CUIT/CUIL/CDI')
    habilitado = models.BooleanField(default=True)
    jubilado = models.BooleanField(default=False)
    fallecido = models.BooleanField(default=False)

    def nombre_completo(self):
        return u'%s %s' % (self.apellidos, self.nombres)

    class Meta:
        verbose_name_plural = 'profesionales'
        ordering = ['apellidos', 'nombres']

    def __str__(self):
        return u'%s %s' % (self.apellidos, self.nombres)

    def save(self, force_insert=False, force_update=False):
        self.apellidos = self.apellidos.upper().strip()
        self.nombres = CapitalizePhrase(self.nombres)
        super(Persona, self).save(force_insert, force_update)


class Titulo(models.Model):
    id = models.AutoField(primary_key=True)
    nombre = CharNullField(max_length=30)

    class Meta:
        ordering = ['nombre']

    def __str__(self):
        return self.nombre


class Zona(models.Model):
    id = models.IntegerField(primary_key=True)
    descripcion = CharNullField(max_length=50)

    class Meta:
        ordering = ['id']

    def __str__(self):
        return u'%d' % self.id
